
#include "i2c_helpers.h"
#include "helpers.h"

#include "inc/hw_memmap.h"

#include <stdbool.h> // required by driverlib
#include <stdint.h>
#include "driverlib/i2c.h"
#include "driverlib/sysctl.h"


void i2c_init(uint32_t i2cBase)
{
    uint32_t sysctlPeriph;
    switch (i2cBase)
    {
        case I2C0_BASE:
            sysctlPeriph = SYSCTL_PERIPH_I2C0;
            break;

        case I2C1_BASE:
            sysctlPeriph = SYSCTL_PERIPH_I2C1;
            break;

        case I2C2_BASE:
            sysctlPeriph = SYSCTL_PERIPH_I2C2;
            break;

        case I2C3_BASE:
            sysctlPeriph = SYSCTL_PERIPH_I2C3;
            break;

        default:
            return;
    }
    // enable I2C module 0
    SysCtlPeripheralEnable(sysctlPeriph);
    while (!SysCtlPeripheralReady(sysctlPeriph))
        ;

    // reset module
    SysCtlPeripheralReset(sysctlPeriph);

    // 400KHZ is not supported by Nanomotion and does not work, so set it to
    // false for all by now.
    I2CMasterInitExpClk(i2cBase, SysCtlClockGet(), false);
}

I2C_retVal i2c_write(uint32_t i2cBase, uint8_t address, char* buf, uint8_t len)
{
    if (len == 0)
    {
        return I2C_ERROR_OTHER;
    }

    I2C_retVal ret = I2C_ERROR_NONE;
    uint32_t start = g_millis;
    I2CMasterSlaveAddrSet(i2cBase, address, false);

    if (len == 1)
    {
        I2CMasterDataPut(i2cBase, buf[0]);
        I2CMasterControl(i2cBase, I2C_MASTER_CMD_SINGLE_SEND);
        ret = i2c_waitForTransmission(i2cBase, start);
    }
    else
    {
        I2CMasterDataPut(i2cBase, buf[0]);
        I2CMasterControl(i2cBase, I2C_MASTER_CMD_BURST_SEND_START);
        ret = i2c_waitForTransmission(i2cBase, start);

        if (ret == I2C_ERROR_NONE)
        {
            // put all except last data byte into I2C buffer
            for (uint8_t i = 1; i < len - 1; ++i)
            {
                I2CMasterDataPut(i2cBase, buf[i]);
                I2CMasterControl(i2cBase, I2C_MASTER_CMD_BURST_SEND_CONT);
                ret = i2c_waitForTransmission(i2cBase, start);
                if (ret != I2C_ERROR_NONE)
                {
                    break;
                }
            }
        }

        // put last data byte into I2C buffer
        I2CMasterDataPut(i2cBase, buf[len - 1]);
        I2CMasterControl(i2cBase, I2C_MASTER_CMD_BURST_SEND_FINISH);
        if (ret == I2C_ERROR_NONE)
        {
            ret = i2c_waitForTransmission(i2cBase, start);
        }
        else
        {
            while (I2CMasterBusy(i2cBase) && (g_millis - start < I2C_TIMEOUT_MS))
            ;
        }
    }

    return ret;
}

I2C_retVal i2c_read(uint32_t i2cBase, uint8_t address, char* buf, uint8_t len)
{
    if (len == 0)
    {
        return I2C_ERROR_OTHER;
    }

    I2C_retVal ret = I2C_ERROR_NONE;
    uint32_t start = g_millis;
    I2CMasterSlaveAddrSet(i2cBase, address, true);

    if (len == 1)
    {
        I2CMasterControl(i2cBase, I2C_MASTER_CMD_SINGLE_RECEIVE);
        ret = i2c_waitForTransmission(i2cBase, start);
        if (ret == I2C_ERROR_NONE)
        {
            buf[0] = I2CMasterDataGet(i2cBase);
        }
    }
    else
    {
        I2CMasterControl(i2cBase, I2C_MASTER_CMD_BURST_RECEIVE_START);
        ret = i2c_waitForTransmission(i2cBase, start);
        if (ret == I2C_ERROR_NONE)
        {
            buf[0] = I2CMasterDataGet(i2cBase);

            // receive all except last byte
            for (uint8_t i = 1; i < len - 1; ++i)
            {
                I2CMasterControl(i2cBase, I2C_MASTER_CMD_BURST_RECEIVE_CONT);
                ret = i2c_waitForTransmission(i2cBase, start);
                if (ret != I2C_ERROR_NONE)
                {
                    break;
                }
                buf[i] = I2CMasterDataGet(i2cBase);
            }
        }

        // receive last byte
        I2CMasterControl(i2cBase, I2C_MASTER_CMD_BURST_RECEIVE_FINISH);
        if (ret == I2C_ERROR_NONE)
        {
            ret = i2c_waitForTransmission(i2cBase, start);
        }
        else
        {
            while (I2CMasterBusy(i2cBase) && (g_millis - start < I2C_TIMEOUT_MS))
            ;
        }
        if (ret == I2C_ERROR_NONE)
        {
            buf[len - 1] = I2CMasterDataGet(i2cBase);
        }
    }

    return ret;
}

I2C_retVal i2c_waitForTransmission(uint32_t i2cBase, uint32_t startTime)
{
    while (I2CMasterBusy(i2cBase) && (g_millis - startTime <= I2C_TIMEOUT_MS))
        ;
    if (g_millis - startTime > I2C_TIMEOUT_MS)
    {
        return I2C_ERROR_TIMEOUT;
    }
    uint32_t err = I2CMasterErr(i2cBase);
    if (!err)
    {
        return I2C_ERROR_NONE;
    }
    if (err & I2C_MASTER_ERR_ADDR_ACK)
    {
        return I2C_ERROR_ADDR_ACK;
    }
    if (err & I2C_MASTER_ERR_DATA_ACK)
    {
        return I2C_ERROR_DATA_ACK;
    }
    return I2C_ERROR_OTHER;
}
